import swarm.simtoolsgui.*;
import swarm.gui.*;
import swarm.space.*;
import swarm.activity.*;
import swarm.defobj.*;
import swarm.objectbase.*;
import swarm.*;
import swarm.analysis.*;

public class ObserverSwarm extends GUISwarmImpl{
	public int displayFrequency,zoomFactor; 
	int times;
	ActionGroup displayActions; // schedule data structs
	Schedule displaySchedule;
	ModelSwarm modelSwarm; // the Swarm we're observing
	Colormap colorMap; // allocate colors
	ZoomRaster worldRaster; // 2d display widget
	Value2dDisplay mapDisplay; // display the heat
	Object2dDisplay beeDisplay; 
	EZGraph resultGraph,scoutsGraph;

	public ObserverSwarm(Zone aZone){
		super(aZone);
		displayFrequency=1;
		zoomFactor=4;
		times=0;
	}
	
	public Object _resultGraphDeath_(Object caller){
		resultGraph.drop();
		resultGraph = null;
		return this;
	}
	public Object _scoutsGraphDeath_(Object caller){
		scoutsGraph.drop();
		scoutsGraph = null;
		return this;
	}
	
	public Object buildObjects(){
		super.buildObjects();
		modelSwarm = new ModelSwarm(Globals.env.globalZone);
		if(modelSwarm==null){
			System.out.println("Can't find the modelSwarm parameters.");
			System.exit(1);
		}
		
		// Now create probe objects on the model and ourselves. This gives a
		// simple user interface to let the user change parameters.
		
		Globals.env.createArchivedProbeDisplay (modelSwarm, "modelSwarm");
		Globals.env.createArchivedProbeDisplay (this, "observerSwarm");
		
		// Instruct the control panel to wait for a button event.
		// We halt here until someone hits a control panel button.
		
		// Now that we're using Probes, the user can set the parameters
		// in the ModelSwarm probe window - we halt here to allow
		// the user to change parameters.
		
		getControlPanel().setStateStopped();
		
		modelSwarm.buildObjects();
		
		colorMap=new ColormapImpl(this);
		colorMap.setColor$ToName((byte)0,"gray");
		colorMap.setColor$ToName((byte)1,"green");
		colorMap.setColor$ToName((byte)2,"yellow");
		colorMap.setColor$ToName((byte)3,"red");
		
		worldRaster=new ZoomRasterImpl(this);
		worldRaster.setColormap(colorMap);
		worldRaster.setZoomFactor(zoomFactor);
		worldRaster.setWidth$Height(
			modelSwarm.getWorld().getSizeX(),
			modelSwarm.getWorld().getSizeY());
		worldRaster.setWindowTitle("Artificial Bee Colony - proposed ABC");
		worldRaster.pack(); // draw the window.
		
		mapDisplay=new Value2dDisplayImpl(this,worldRaster,colorMap,modelSwarm.getMap());
		try {
			beeDisplay = new Object2dDisplayImpl(
				this,
				worldRaster,
				modelSwarm.getWorld(),
				new Selector(Class.forName("Bee"), "drawSelfOn", false));
		} catch (Exception e) {
			System.out.println ("Exception: " + e.getMessage ());
			System.exit(1);
		}

		//fitness graph
		resultGraph = new EZGraphImpl(this, "Model Fitness", "generations",
				"fitness", "resultGraph");
		try {
			resultGraph.enableDestroyNotification$notificationMethod(this,
					new Selector(getClass(), "_resultGraphDeath_", false));
		} catch (Exception e) {
			System.err.println("Exception _resultGraphDeath_: "
					+ e.getMessage());
		}
		
		try {
			resultGraph.createSequence$withFeedFrom$andSelector(
					"AverageFitness", this, new Selector(this.getClass(),
							"getAvgFitness", false));
			resultGraph
					.createSequence$withFeedFrom$andSelector("MaxFitness",
							this, new Selector(this.getClass(),
									"getMaxFitness", false));
		} catch (Exception e) {
			System.err.println("Exception: " + e.getMessage());
		}
		//scouts Graph
		scoutsGraph = new EZGraphImpl(this, "Scouts work", "generations",
				"scouts work", "scoutsGraph");

		try {
			scoutsGraph.enableDestroyNotification$notificationMethod(this,
					new Selector(getClass(), "_scoutsGraphDeath_", false));
		} catch (Exception e) {
			System.err.println("Exception _scoutsGraphDeath_: "
					+ e.getMessage());
		}
		try {
			scoutsGraph.createSequence$withFeedFrom$andSelector("scouts work", this, new Selector(this.getClass(),"getScoutsWork",false));
		}catch (Exception e) {
			System.err.println("Exception: " + e.getMessage());
		}

		return this;
	}
	
	public Object buildActions(){
		super.buildActions();
		modelSwarm.buildActions();
		displayActions=new ActionGroupImpl(this);
		try {
			displayActions.createActionTo$message(mapDisplay,
				new Selector(mapDisplay.getClass(),"display",false));
			displayActions.createActionTo$message(beeDisplay,
				new Selector(beeDisplay.getClass(),"display",false));
			displayActions.createActionTo$message(worldRaster,
				new Selector(worldRaster.getClass(),"drawSelf",false));
			displayActions.createActionTo$message(this,new Selector(this.getClass(),"showStats",false));
			displayActions.createActionTo$message(this,new Selector(this.getClass(),"tmaxCheck",false));
			displayActions.createActionTo$message(
				getActionCache(),
				new Selector(getActionCache().getClass(),"doTkEvents",false));
			displayActions.createActionTo$message(modelSwarm,
				new Selector(modelSwarm.getClass(),"step",false));
		} catch (Exception e) {
			System.out.println ("Exception: " + e.getMessage ());
			System.exit(1);
		}
		
		displaySchedule = new ScheduleImpl(this,displayFrequency);
		displaySchedule.at$createAction(0,displayActions);
		
		return this;
    }
	
	public Activity activateIn(Swarm context){
    	super.activateIn(context);
		modelSwarm.activateIn(this);
		displaySchedule.activateIn(this);
		return getActivity();
	}	
	
	public Object showStats(){
		if(resultGraph!=null){
			resultGraph.step(); // step the result Graph
		}
		if(scoutsGraph!=null){
			scoutsGraph.step();
		}
		return this;
	}
	
	public Object tmaxCheck(){
		if ((modelSwarm.getT()>=modelSwarm.tmax-1)&&(modelSwarm.tmax!=0)){
			getControlPanel().setStateStopped();
		}
		return this;
	}
	
	public double getAvgFitness(){
		return modelSwarm.getAvgFitness();
	}
	
	public double getMaxFitness(){
		return modelSwarm.getMaxFitness();
	}
	
	public int getScoutsWork(){
		return modelSwarm.getScoutsWork();
	}
	
	public double getAvgS(){
		return modelSwarm.getAvgS();
	}
	
	public double get2ndS(){
		return modelSwarm.get2ndS();
	}
	
	public double getMaxS(){
		return modelSwarm.getMaxS();
	}
}
